#include <cstdio>
#include <iostream>
#include <string>

// RevTo107 - converts Rev WLDs to 1.07 WLDs.

int main (int argc, char* argv [])
{
	if (2 > argc)
	{
		std::cout << "Usage: RevTo107.exe WLDFile.wld\n";
		return 0;
	}

	std::string strFilePath(argv[1]);
	size_t extPoint = strFilePath.find_last_of('.');
	if (std::string::npos == extPoint)
	{
		std::cout << "Bad file path\n";
	}
	std::string strFileNameNoExt = strFilePath.substr(0, extPoint);
	std::string strFileNameSave = strFileNameNoExt + std::string("107.wld");

	// Load file
	FILE * pFile;
	pFile = fopen(strFilePath.c_str(), "rb");
	if (pFile == NULL)
	{
		std::cout << "Cannot open file\n";
		return 0;		
	}

	fseek(pFile, 0, SEEK_END);
	size_t fileSize = ftell(pFile);
	rewind(pFile);

	if (16 >= fileSize)
	{
		std::cout << "File is empty\n";
		fclose(pFile);
		return 0;
	}

	unsigned char *pBuffer = new unsigned char[fileSize];
	unsigned char *pCorrected = new unsigned char[fileSize - 16];
	size_t readBytes = fread(pBuffer, 1, fileSize, pFile);
	if (fileSize != readBytes)
	{
		std::cout << "File read error\n";
		fclose(pFile);
		return 0;
	}
	fclose (pFile);

	// Analyze buffer
	const char* chunkIDLDRB = "LDRB";
	const char* chunkIDDPOS = "DPOS";
	const char* chunkIDDICT = "DICT";
	size_t leaderboardPos = 0;
	for (size_t posID = 0; posID < fileSize - 4; ++posID)
	{
		if (0 != memcmp(pBuffer + posID, chunkIDLDRB, 4))
			continue;

		leaderboardPos = posID;
		memcpy((void*)pCorrected, (const void*)pBuffer, leaderboardPos);
		break;
	}
	if (0 == leaderboardPos)
	{
		std::cout << "Unsupported version\n";
		return 0;
	}
	size_t leaderboardPos2 = 0;
	for (size_t posID = leaderboardPos + 8; posID < fileSize - 4; ++posID)
	{
		if (0 != memcmp(pBuffer + posID, chunkIDLDRB, 4))
			continue;

		leaderboardPos2 = posID;
		memcpy(pCorrected + leaderboardPos, pBuffer + leaderboardPos + 8, leaderboardPos2 - leaderboardPos - 8);
		memcpy(pCorrected + leaderboardPos2 - 8, pBuffer + leaderboardPos2 + 8, fileSize - leaderboardPos2 - 8);
		break;
	}
	if (0 == leaderboardPos2)
	{
		std::cout << "Unsupported version\n";
		return 0;
	}
	size_t dictPosPos = 0;
	size_t dictPosPos2 = 0;
	for (size_t posID = 0; posID < fileSize - 20; ++posID)
	{
		if (0 != memcmp(pCorrected + posID, chunkIDDPOS, 4))
			continue;

		if (0 == dictPosPos)
		{
			dictPosPos = posID;
		}
		else
		{
			dictPosPos2 = posID;
			break;
		}
	}
	if ((0 == dictPosPos) || (0 == dictPosPos2))
	{
		std::cout << "Unsupported version\n";
		return 0;
	}

	size_t dictPos = 0;
	size_t dictPos2 = 0;
	for (size_t posID = 0; posID < fileSize - 20; ++posID)
	{
		if (0 != memcmp(pCorrected + posID, chunkIDDICT, 4))
			continue;

		if (0 == dictPos)
		{
			dictPos = posID;
		}
		else
		{
			dictPos2 = posID;
			break;
		}
	}
	if ((0 == dictPos) || (0 == dictPos2))
	{
		std::cout << "Unsupported version\n";
		return 0;
	}

	(unsigned short &)(*(pCorrected + dictPosPos + 4)) = (unsigned short)dictPos;
	(unsigned short &)(*(pCorrected + dictPosPos2 + 4)) = (unsigned short)dictPos2;

	// Save file
	FILE * pFile2;
	pFile2 = fopen(strFileNameSave.c_str(), "wb");
	if (pFile2 == NULL)
	{
		std::cout << "Cannot create file\n";
		return 0;		
	}

	size_t writtenBytes = fwrite(pCorrected, 1, fileSize - 16, pFile2);
	if (fileSize - 16 != writtenBytes)
	{
		std::cout << "Write error\n";
		fclose(pFile2);
		return 0;
	}

	fclose(pFile2);
	return 0;
}